const axios = require('axios');
const { SUBSCRIPTION_BASE_URL, CLOUD_TIERING_NAMESPACE, TIERING_SERVICE_NAME_SUBSCRIPTION } = require('../utils/consts');
const { getTenancyServiceToken } = require('./auth0');
const { listSubscriptionResponse } = require('../joi-schemas/subscription-schemas');
const { validateResponse } = require('../utils/schema-validation');

async function listSubscriptionsOfAccount(accountId) {
    const { data } = await axios.get(`${SUBSCRIPTION_BASE_URL}/${accountId}`, {
        headers: {
            authorization: getTenancyServiceToken(),
            'X-User-Token': CLOUD_TIERING_NAMESPACE.get('userToken')
        }
    });
    // data is [{subscriptionId, cloudProvider, name, status, serviceName}]
    validateResponse(data, listSubscriptionResponse);
    return data;
}

async function listAllSubscriptionsForService() {
    const { data } = await axios.get(`${SUBSCRIPTION_BASE_URL}/by-service-name?serviceName=${TIERING_SERVICE_NAME_SUBSCRIPTION}`, {
        headers: {
            authorization: getTenancyServiceToken()
        }
    });
    // data is [{subscriptionId, cloudProvider, name, status, serviceName}]
    validateResponse(data, listSubscriptionResponse);
    return data;
}

module.exports = { listSubscriptionsOfAccount, listAllSubscriptionsForService };
