const randomize = require('randomatic');
const os = require('os');
const Agenda = require('agenda');
const config = require('config');

const getLogger = require('../utils/logger/logger');
const {
    AGENDA_PROCESS_EVERY, AGENDA_JOB_NAME_SAAS_METERING, AGENDA_JOB_NAME_INCREASE_LICENSE_CAPACITY, CLOUD_TIERING_NAMESPACE
} = require('./consts');
const { collectChargingInfo } = require('../operations/charging-operations');
const { updateLicenses } = require('../operations/license-operations');

const logger = getLogger();

function initAgenda(db) {
    const agenda = new Agenda({
        name: `${os.hostname}-${process.pid}`,
        processEvery: AGENDA_PROCESS_EVERY,
        mongo: db,
        sort: { nextRunAt: 1 }
    });

    return new Promise((resolve, reject) => {
        agenda.on('ready', () => {
            agenda.define(AGENDA_JOB_NAME_SAAS_METERING, { lockLifetime: 10000 }, async (job, done) => {
                CLOUD_TIERING_NAMESPACE.run(async () => {
                    try {
                        // generate a requestId and task for logging
                        const requestId = `CT-${randomize('Aa0', 8)}`;
                        CLOUD_TIERING_NAMESPACE.set('requestId', requestId);
                        CLOUD_TIERING_NAMESPACE.set('task', 'charging');
                        const chargingTime = new Date().toISOString();
                        logger.info('Scheduled MarketPlace Metering Billing', chargingTime);
                        await collectChargingInfo(chargingTime);
                    } catch (err) {
                        logger.error('Error running agenda with error', err);
                    } finally {
                        done();
                    }
                });
            });
            agenda.every(config.get('agenda.charging-interval'), AGENDA_JOB_NAME_SAAS_METERING);

            agenda.define(AGENDA_JOB_NAME_INCREASE_LICENSE_CAPACITY, { lockLifetime: 60000 }, async (job, done) => {
                try {
                    logger.info('Scheduled license capacity updater');
                    await updateLicenses();
                } catch (err) {
                    logger.error('Error running agenda with error', err);
                } finally {
                    done();
                }
            });
            agenda.every(config.get('agenda.license-capacity-increase-interval'), AGENDA_JOB_NAME_INCREASE_LICENSE_CAPACITY);

            agenda.start();
            resolve();
        });

        agenda.on('error', err => {
            logger.error('Got agenda error', err);
            reject(err);
        });
    });
}

module.exports = initAgenda;
