const { createNamespace } = require('cls-hooked');
const config = require('config');

const SERVICE_REGION = 'us-east-1';

const AGENDA_PROCESS_EVERY = '10 seconds';
const AGENDA_JOB_NAME_SAAS_METERING = 'MARKETPLACE_METERING';
const AGENDA_JOB_NAME_INCREASE_LICENSE_CAPACITY = 'INCREASE_LICENSE_CAPACITY';

const SUBSCRIBED = 'subscribed';
const UNSUBSCRIBED = 'unsubscribed';
const FAILED_SUBSCRIPTION = 'failed';
const UNSUBSCRIBE_PENDING = 'unsubscribe-pending';
const PENDING_SUBSCRIPTION = 'pending';
const TIERING_SERVICE_NAME_SUBSCRIPTION = config.get('charging-service-name') || 'cloud-tiering'; // for tests
const CHARGING_DIMENSION = config.get('charging-dimension');
const FREE_TRIAL_DAYS = 30;


const DB_CONNECTION_STRING = process.env.DB_CONNECTION_STRING || config.get('db-connection-string');

// Endpoints
const MULTI_TENANCY_ENDPOINT = process.env.MULTI_TENANCY_ENDPOINT ? `http://${process.env.MULTI_TENANCY_ENDPOINT}` : config.get('services-urls.tenancy');
const AGENTS_MGMT_ENDPOINT = process.env.AGENTS_MGMT_ENDPOINT ? `http://${process.env.AGENTS_MGMT_ENDPOINT}` : config.get('services-urls.agents');
const CLOUD_MANAGER_ENDPOINT = process.env.CLOUD_MANAGER_ENDPOINT ? `http://${process.env.CLOUD_MANAGER_ENDPOINT}` : config.get('services-urls.occm');
const SUBSCRIPTION_ENDPOINT = process.env.SUBSCRIPTION_ENDPOINT ? `https://${process.env.SUBSCRIPTION_ENDPOINT}` : config.get('services-urls.subscription');
const CHARGING_ENDPOINT = process.env.CHARGING_ENDPOINT ? `http://${process.env.CHARGING_ENDPOINT}` : config.get('services-urls.charging');

const DISCOVER_LOCATION = 'CLOUD_TIERING';
const CLOUD_MANAGER_API_PREFIX = '/occm/api';
const CLOUD_MANAGER_CLUSTERS_PATH_PREFIX = '/onprem/working-environments';
const CLOUD_MANAGER_CLUSTER_METADATA_PATH_PREFIX = '/onprem/metadata/cluster-info';
const CLOUD_MANAGER_AGGREGATES_PATH_PREFIX = '/onprem/aggregates';
const CLOUD_MANAGER_VOLUMES_PATH_PREFIX = '/onprem/volumes';
const CLOUD_MANAGER_OBJECT_STORE_PATH_PREFIX = '/onprem/object-store';
const CLOUD_MANAGER_AZURE_METADATA_REGIONS = '/azure/vsa/metadata/regions';
const CLOUD_MANAGER_AZURE_METADATA_SUBSCRIPTIONS = '/azure/vsa/metadata/associated-subscriptions';
const CLOUD_MANAGER_OBJECT_STORE_CONFIG_PATH_PREFIX = `${CLOUD_MANAGER_OBJECT_STORE_PATH_PREFIX}/config`;
const CLOUD_MANAGER_AGGREGATE_ATTACH_PATH_PREFIX = `${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/object-store-attach`;
const CLOUD_MANAGER_FABRIC_POOL_LICENSE_PATH_SUFFIX = 'fabric-pool-license';
const CLOUD_MANAGER_INACTIVE_DATA_REPORTING_PATH_SUFFIX = 'enable-cold-data-reporting';
const CLOUD_MANAGER_CONFIGURE_FP_RESOURCES_PATH_SUFFIX = 'fabric-pool-configure';
const CLOUD_MANAGER_CONFIGURE_AZURE_FP_RESOURCES_PATH_SUFFIX = 'azure-fabric-pool-configure';

const OCCM_BASE_URL = `${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}`;

const SUBSCRIPTION_API_PREFIX = '/subscriptions';
const SUBSCRIPTION_BASE_URL = `${SUBSCRIPTION_ENDPOINT}${SUBSCRIPTION_API_PREFIX}`;

const CHARGING_API_PREFIX = '/charging';
const CHARGING_BASE_URL = `${CHARGING_ENDPOINT}${CHARGING_API_PREFIX}`;


const JWT_TOKEN_AUTH0 = config.get('auth0-url');
const JWT_TOKEN_API_AUTH0_AUDIENCE = 'https://cloudmanager.cloud.netapp.com';
const JWR_TOKEN_TENANCY_AUDIENCE = 'https://api.cloud.netapp.com';
const CLIENT_ID = process.env.CLIENT_ID || config.get('client_id');
const CLIENT_SECRET = process.env.CLIENT_SECRET || config.get('client_secret');


const IPA_BASE_URL = 'https://support.netapp.com/ServiceGW';
const IPA_SIGN_PATH = '/signMessage';

const SIGNED_SERVICE_USER_NAME = process.env.SIGNED_SERVICE_USER_NAME || config.get('signed_service_user_name');
const SIGNED_SERVICE_PASSWORD = process.env.SIGNED_SERVICE_PASSWORD || config.get('signed_service_password');
const NAME_SPACE_NAME = 'cloud-tiering';
const CLOUD_TIERING_NAMESPACE = createNamespace(NAME_SPACE_NAME);// each application wanting to use continuation-local values should create its own namespace

const API_ROOT = '/api';
const GRAPHQL_ROOT = '/graphql';
const API_PATH_VOLUMES = '/volumes';
const API_PATH_TENANTS = '/tenants';
const API_PATH_CLUSTERS = '/clusters';
const API_PATH_ACCOUNTS = '/tenancy/external/account';
const API_PATH_AGENTS = '/agents-mgmt/external/agent';
const API_PATH_RESOURCES = '/tenancy/external/resource';
const API_PATH_RESOURCES_INTERNAL = '/tenancy/internal/resource';
const API_PATH_SAAS_REGISTRATION = '/saas-registration';
const API_PATH_HEALTH = '/health';

const SKIP_AUTHENTICATION_REQUESTS_PATH = [
    API_PATH_SAAS_REGISTRATION, API_PATH_HEALTH
];

const SAVING_OPPORTUNITY_ESTIMATION_RATIO = 0.7;

const AWS = 'AWS';
const AZURE = 'AZURE';
const cloudProviders = {
    AZURE: 'azure',
    AWS: 'aws'
};

const SNAPSHOT_ONLY = 'snapshot_only';
const AUTO = 'auto';
const NONE = 'none';

const occmProviderToCloudProviders = {
    aws_s3: AWS,
    azure_cloud: AZURE
};

const storageClasses = {
    [AWS]: {
        ia: 'Standard-Infrequent Access', // from occm to UI
        standard_ia: 'ia', // from tiering to occm
        normal: 'Standard', // from occm to UI
        standard: 'normal', // from tiering to occm,
        hot: 'hot'
    },
    [AZURE]: {
        normal: 'Hot',
        cool: 'Cold'
    }
};


module.exports = {
    SERVICE_REGION,
    SUBSCRIBED,
    UNSUBSCRIBED,
    MULTI_TENANCY_ENDPOINT,
    CLOUD_MANAGER_ENDPOINT,
    CLOUD_MANAGER_CLUSTERS_PATH_PREFIX,
    CLOUD_MANAGER_CLUSTER_METADATA_PATH_PREFIX,
    API_ROOT,
    API_PATH_VOLUMES,
    API_PATH_CLUSTERS,
    API_PATH_TENANTS,
    API_PATH_SAAS_REGISTRATION,
    API_PATH_RESOURCES,
    API_PATH_RESOURCES_INTERNAL,
    IPA_SIGN_PATH,
    SKIP_AUTHENTICATION_REQUESTS_PATH,
    JWT_TOKEN_API_AUTH0_AUDIENCE,
    API_PATH_ACCOUNTS,
    CLOUD_MANAGER_AGGREGATES_PATH_PREFIX,
    CLOUD_MANAGER_VOLUMES_PATH_PREFIX,
    CLOUD_MANAGER_FABRIC_POOL_LICENSE_PATH_SUFFIX,
    CLOUD_MANAGER_INACTIVE_DATA_REPORTING_PATH_SUFFIX,
    CLOUD_MANAGER_OBJECT_STORE_CONFIG_PATH_PREFIX,
    CLOUD_MANAGER_AGGREGATE_ATTACH_PATH_PREFIX,
    IPA_BASE_URL,
    API_PATH_AGENTS,
    CLOUD_MANAGER_API_PREFIX,
    AGENDA_PROCESS_EVERY,
    AGENDA_JOB_NAME_SAAS_METERING,
    AGENDA_JOB_NAME_INCREASE_LICENSE_CAPACITY,
    CLOUD_MANAGER_CONFIGURE_FP_RESOURCES_PATH_SUFFIX,
    AGENTS_MGMT_ENDPOINT,
    GRAPHQL_ROOT,
    DISCOVER_LOCATION,
    DB_CONNECTION_STRING,
    API_PATH_HEALTH,
    SAVING_OPPORTUNITY_ESTIMATION_RATIO,
    JWR_TOKEN_TENANCY_AUDIENCE,
    CLIENT_ID,
    CLIENT_SECRET,
    JWT_TOKEN_AUTH0,
    SIGNED_SERVICE_USER_NAME,
    SIGNED_SERVICE_PASSWORD,
    NAME_SPACE_NAME,
    CLOUD_TIERING_NAMESPACE,
    OCCM_BASE_URL,
    CLOUD_MANAGER_AZURE_METADATA_REGIONS,
    CLOUD_MANAGER_AZURE_METADATA_SUBSCRIPTIONS,
    CLOUD_MANAGER_CONFIGURE_AZURE_FP_RESOURCES_PATH_SUFFIX,
    storageClasses,
    cloudProviders,
    SNAPSHOT_ONLY,
    AUTO,
    NONE,
    AWS,
    AZURE,
    SUBSCRIPTION_ENDPOINT,
    TIERING_SERVICE_NAME_SUBSCRIPTION,
    CHARGING_ENDPOINT,
    FAILED_SUBSCRIPTION,
    UNSUBSCRIBE_PENDING,
    PENDING_SUBSCRIPTION,
    occmProviderToCloudProviders,
    FREE_TRIAL_DAYS,
    CHARGING_DIMENSION,
    SUBSCRIPTION_BASE_URL,
    CHARGING_BASE_URL,
    CHARGING_API_PREFIX,
    SUBSCRIPTION_API_PREFIX
};
