const { isLicensedAccountInternal } = require('../../src/operations/subscription-operations');
const { updateLicense } = require('../../src/operations/license-operations');
const { getAllClustersInAccountInternal } = require('../../src/operations/multi-tenancy-operation');
const { licenseStatus } = require('../../src/lib/cloud-manager/clusters');
const createLicenseAndApply = require('../../src/operations/ipa-operations');


jest.mock('../../src/operations/subscription-operations');
jest.mock('../../src/operations/multi-tenancy-operation');
jest.mock('../../src/lib/cloud-manager/clusters');
jest.mock('../../src/operations/ipa-operations');

const clusters = [{
    name: 'AFF-1',
    publicId: 'eda35168-d036-4cce-9cc8-ca916ec36324',
    ontapClusterProperties: {
        ontapVersion: '9.5',
        clusterAddress: '192.30.0.1',
        nodeModels: ['A200'],
        clusterUuid: '10bec0b2-7c82-404f-868a-424cf5158b80',
        lifs: []
    },
    capacityFeatures: {
        objectStoreConfigExtended: null,
        performanceTierUsedCapacity: {
            size: 97.47,
            unit: 'TB'
        },
        inactiveUserData: {
            size: 67.74,
            unit: 'TB'
        },
        rawCapacity: {
            size: 150,
            unit: 'TB'
        }
    },
    status: {
        status: 'ON'
    }
}, {
    name: 'AFF-2',
    publicId: 'eda35168-d036-4cce-9cc8-ca916ec36325',
    ontapClusterProperties: {
        ontapVersion: '9.5',
        clusterAddress: '192.30.0.1',
        nodeModels: ['A200'],
        clusterUuid: '10bec0b2-7c82-404f-868a-424cf5158b80',
        lifs: []
    },
    capacityFeatures: {
        objectStoreConfigExtended: null,
        performanceTierUsedCapacity: {
            size: 97.47,
            unit: 'TB'
        },
        inactiveUserData: {
            size: 67.74,
            unit: 'TB'
        },
        rawCapacity: {
            size: 150,
            unit: 'TB'
        }
    },
    status: {
        status: 'ON'
    }
}];

function licenseStatusObject(current, max) {
    return {
        licenseInformation: {
            currentCapacity: { size: current, unit: 'GB' },
            maxCapacity: { size: max, unit: 'TB' },
            serialNumber: '12345678'
        }
    };
}

describe('update license.json', () => {
    beforeEach(() => {
        createLicenseAndApply.mockClear();
    });

    test('should do nothing when account is not licensed', async () => {
        isLicensedAccountInternal.mockReturnValue(false);
        expect(await updateLicense('account-123')).toBeUndefined();
    });

    test('should not update cluster\'s license when license capacity ratio is under the threshold', async () => {
        isLicensedAccountInternal.mockReturnValue(true);
        getAllClustersInAccountInternal.mockReturnValue(clusters);
        licenseStatus.mockReturnValue(licenseStatusObject(1, 10));
        expect(await updateLicense('account-123')).toEqual([]);
        expect(createLicenseAndApply).not.toHaveBeenCalled();
    });

    test('should update cluster\'s license  when license capacity ratio is above threshold', async () => {
        isLicensedAccountInternal.mockReturnValue(true);
        getAllClustersInAccountInternal.mockReturnValue(clusters);
        licenseStatus.mockReturnValue(licenseStatusObject(8, 10));
        expect(await updateLicense('account-123')).toEqual([]);
        expect(createLicenseAndApply).toHaveBeenCalledTimes(2);
    });

    test('should skip when update of one license was failed', async () => {
        isLicensedAccountInternal.mockReturnValue(true);
        getAllClustersInAccountInternal.mockReturnValue(clusters);
        licenseStatus.mockReturnValue(licenseStatusObject(8, 10));
        createLicenseAndApply.mockRejectedValueOnce(new Error('error'))
            .mockResolvedValue();
        expect(await updateLicense('account-123')).toEqual([]);
        expect(createLicenseAndApply).toHaveBeenCalledTimes(2);
    });
});
