const nock = require('nock');
const {
    CLOUD_MANAGER_AZURE_METADATA_SUBSCRIPTIONS, CLOUD_MANAGER_ENDPOINT, CLOUD_MANAGER_API_PREFIX, CLOUD_MANAGER_AZURE_METADATA_REGIONS
} = require('../../../src/utils/consts');
const getLogger = require('../../../src/utils/logger/logger');

const logger = getLogger('simulator');

const azureMetadataScope = nock(CLOUD_MANAGER_ENDPOINT)
    .log(evt => logger.info(evt))
    // associated subscriptions
    .get(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AZURE_METADATA_SUBSCRIPTIONS}`)
    .delay(100)
    .reply(() => [200, JSON.stringify([{
        subscriptionId: '1b05f27b-7dd6-43eb-9164-e1903d7a4321',
        name: 'OCCM QA1',
        default: true
    },
    {
        subscriptionId: '1b05f27b-7dd6-43eb-9164-e1903d7a4337',
        name: 'OCCM DEV',
        default: false
    }])])

    // azure regions
    .get(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AZURE_METADATA_REGIONS}`)
    .query(query => query.subscriptionId)
    .delay(100)
    .reply(() => [200, [
        {
            displayName: 'East Asia',
            name: 'eastasia',
            vnets: null
        },
        {
            displayName: 'Southeast Asia',
            name: 'southeastasia',
            vnets: null
        },
        {
            displayName: 'Central US',
            name: 'centralus',
            vnets: null
        },
        {
            displayName: 'East US',
            name: 'eastus',
            vnets: null
        },
        {
            displayName: 'East US 2',
            name: 'eastus2',
            vnets: null
        },
        {
            displayName: 'West US',
            name: 'westus',
            vnets: null
        },
        {
            displayName: 'North Central US',
            name: 'northcentralus',
            vnets: null
        },
        {
            displayName: 'South Central US',
            name: 'southcentralus',
            vnets: null
        },
        {
            displayName: 'North Europe',
            name: 'northeurope',
            vnets: null
        },
        {
            displayName: 'West Europe',
            name: 'westeurope',
            vnets: null
        },
        {
            displayName: 'Japan West',
            name: 'japanwest',
            vnets: null
        },
        {
            displayName: 'Japan East',
            name: 'japaneast',
            vnets: null
        },
        {
            displayName: 'Brazil South',
            name: 'brazilsouth',
            vnets: null
        },
        {
            displayName: 'Australia East',
            name: 'australiaeast',
            vnets: null
        },
        {
            displayName: 'Australia Southeast',
            name: 'australiasoutheast',
            vnets: null
        },
        {
            displayName: 'Canada Central',
            name: 'canadacentral',
            vnets: null
        },
        {
            displayName: 'Canada East',
            name: 'canadaeast',
            vnets: null
        },
        {
            displayName: 'UK South',
            name: 'uksouth',
            vnets: null
        },
        {
            displayName: 'UK West',
            name: 'ukwest',
            vnets: null
        },
        {
            displayName: 'West Central US',
            name: 'westcentralus',
            vnets: null
        },
        {
            displayName: 'West US 2',
            name: 'westus2',
            vnets: null
        },
        {
            displayName: 'Korea Central',
            name: 'koreacentral',
            vnets: null
        },
        {
            displayName: 'Korea South',
            name: 'koreasouth',
            vnets: null
        },
        {
            displayName: 'France Central',
            name: 'francecentral',
            vnets: null
        }
    ]]);

azureMetadataScope.persist(true);
