const nock = require('nock');
const { validateSchema } = require('../../../src/utils/schema-validation');
const chargingRequest = require('../../../src/joi-schemas/charging-schemas');
const {
    CHARGING_ENDPOINT, CHARGING_API_PREFIX
} = require('../../../src/utils/consts');
const { escapeForwardSlash } = require('../simulator-utils');
const getLogger = require('../../../src/utils/logger/logger');

const logger = getLogger('simulator');

const chargingServiceScope = nock(CHARGING_ENDPOINT)
    .log(evt => logger.info(evt))
    // create subscription - fake API for testing
    .post(escapeForwardSlash(`${CHARGING_API_PREFIX}`),
        body => {
            try {
                validateSchema(body, chargingRequest);
                return true;
            } catch (err) {
                logger.error('CHARGING ERROR', err);
                return false;
            }
        })
    .delay(100)
    .reply(200, (uri, requestBody) => {
        const charging = JSON.parse(requestBody);
        return [200, JSON.stringify(charging)];
    });

chargingServiceScope.persist(true);
